/* used for benchmarking the eventqueue and other megasquirtavr stuff
 * Author: Marcell Gal
 * license: GPL
 */


   /*!
   * \addtogroup g_debug Debug Routines
   */
/*@{*/

#include "benchmark.h"
#include "eventqueue.h"
#include "global.h"
#include "lcd.h"
#include "ve.h"
#include "helpers.h"
#include "comm.h"
#include "kb.h"
// #include <math.h>
// #include <float.h>

#include "arithm.c"

// menu puts some stuff here for sending
uint16_t tmp_for_comm[TMP_FOR_COMM_LEN];

struct debug_t debug;

void debug_init(void)
{
	debug.megatune_table = ve.ref_table;
	debug.auto_dump_freq = COMM_FREQ_DEFAULT;
	debug.auto_dump_mask = 0xff;
#if 0	// gcc zeroes SRAM for us
	debug.forced_pw = 0;
	debug.forced_adv = 0;
	debug.bootl = 0;
	debug.disable_flags = 0;
#endif
}

#ifdef BENCHMARK
struct benchstats_t benchstats;
uint16_t evq_histogram[EVQHISTOGRAM_BUCKETS];
uint16_t ic3_histogram[IC3HISTOGRAM_BUCKETS];

void benchmark_init(void)
{ // is this needed at all? Or does it happen around .init4 anyway?
	int8_t i;

	for(i=sizeof(struct benchstats_t)-1; i>=0; i--){
		((char*)(&benchstats))[i] = 0;
	}
	/*for(i = EVQHISTOGRAM_BUCKETS - 1; i>=0; i--){
		evq_histogram[i] = 0;
	}*/
}

/* track event latencies and other problems */
__inline__ void benchmark_histogram(uint16_t *histogramptr, uint16_t diff, uint8_t buckets)
{
	/* normally 0 <= diff <OCR_SAFETY, 
	OCR_SAFETY + 1 is a collector for anything higher */

	benchstats.eventnum++;	// redundant, equals the sum of the arrays
	if(diff >= buckets){
		histogramptr[buckets-1] ++; // for too much diff we only get an aggregate
		return;
	}
	histogramptr[diff] ++;
}

/* used for sending "\r\n" + (OCR_SAFETY + 2) words through the serial port 
	More readable than sending little-endian binary data
*/
uint8_t evq_latency_comm(uint8_t i)
{
	uint16_t	t;
	uint8_t	ind;
	if(i==0) return '\r';
	if(i==1) return '\n';
	i-=2;
	ind=i>>3;
	t = evq_histogram[ind]; // each word sent in 8bytes
	i &= 7;
	switch(i){	// 2 hexabytes and 2 spaces
		case 0: return ' ';
		case 1: return 'E';	// hex_hi(ind);
		case 2: return hex_lo(ind);
		case 3: return '=';
		case 4: return hex_hi(t>>8);
		case 5: return hex_lo(t>>8);
		case 6: return hex_hi(t&0x00FF);
		case 7: return hex_lo(t&0x00FF);
		default: return 'e';
	}
}
/* similar readable per-word sending as in latency_comm 
These could be consolidated using the private attribute in the 
command (and pushfuncpar) see below.
*/
uint8_t benchstats_comm(uint8_t i)
{
//	if(i < BENCHSTATS_LENGTH) return ((uint8_t *)&benchstats)[i]; // unreadable
	uint16_t	t;
	uint8_t	ind;
	if(i==0) return '\r';
	if(i==1) return '\n';
	i-=2;
	ind=i>>3;
	t = ((uint16_t *)&benchstats)[ind]; // each word sent in 8bytes
	i &= 7;
	switch(i){	// 2 hexabytes and 2 spaces
		case 0: return ' ';
		case 1: return 'B';	// hex_hi(ind);
		case 2: return hex_lo(ind);
		case 3: return '=';
		case 4: return hex_hi(t>>8);
		case 5: return hex_lo(t>>8);
		case 6: return hex_hi(t&0x00FF);
		case 7: return hex_lo(t&0x00FF);
		default: return 'e';
	}
	return 'Q';
}
/* guess what: */
uint8_t ic3_latency_comm(uint8_t i)
{
	uint16_t	t;
	uint8_t	ind;
	if(i==0) return '\r';
	if(i==1) return '\n';
	i-=2;
	ind=i>>3;
	t = ic3_histogram[ind]; // each word sent in 8bytes
	i &= 7;
	switch(i){	// 2 hexabytes and 2 spaces
		case 0: return ' ';
		case 1: return 'I';	// hex_hi(ind);
		case 2: return hex_lo(ind);
		case 3: return '=';
		case 4: return hex_hi(t>>8);
		case 5: return hex_lo(t>>8);
		case 6: return hex_hi(t&0x00FF);
		case 7: return hex_lo(t&0x00FF);
		default: return 'e';
	}
}

#endif // BENCHMARK

uint8_t gen_wcomm(uint8_t i)
{
	uint16_t	t;
	uint8_t	ind;
	if(i==0) return '\r';
	if(i==1) return '\n';
	i-=2;
	ind=i>>3;
	t = (cmd_queue[cmd_queue_head].private)[ind]; // each word sent in 8bytes
	i &= 7;
	switch(i){	// 2 hexabytes and 2 spaces
		case 0: return ' ';
		case 1: return cmd_queue[cmd_queue_head].privch.a;
		case 2: return hex_lo(ind);
		case 3: return '=';
		case 4: return hex_hi(t>>8);
		case 5: return hex_lo(t>>8);
		case 6: return hex_hi(t&0x00FF);
		case 7: return hex_lo(t&0x00FF);
		default: return 'e';
	}
}

static inline void benchstats_sendsnapshot(void)
{
#ifdef LCD 
		if(debug.auto_dump_mask & 1) pushfunc(&lcd_cache2comm, LCD_CHARS + 2*LCD_LINES); 
#endif
		if(debug.auto_dump_mask & 4) pushfuncpar(&gen_wcomm, 76, (void*)&h1); 
//		if(debug.auto_dump_mask & 8) pushfuncpar(&gen_wcomm, 76, (void*)&h2); 
#ifdef PWANAL
		if(debug.auto_dump_mask & 0x10) pushfunc(&pw2comm,  16 ); 
#endif
#ifdef BENCHMARK
		if(debug.auto_dump_mask & 0x20) pushfunc(&benchstats_comm, 4*BENCHSTATS_LENGTH + 2);
		if(debug.auto_dump_mask & 0x40) pushfunc(&evq_latency_comm, EVQHISTOGRAM_COMMBYTES); 
		if(debug.auto_dump_mask & 0x80) pushfuncpar2(&gen_wcomm, IC3HISTOGRAM_COMMBYTES, (uint8_t *)&ic3_histogram, 'I'); 
#endif
#ifdef PS2KEYBOARD
		if(debug.auto_dump_mask & 0x02) pushfunc(&ps2_stat_comm, PS2_STAT_COMMBYTES); 

#endif
}

__inline__ void benchstats_maysend(void)
{
	if(debug.auto_dump_freq == 0)
		return;
		
	if(++debug.auto_dump_cnt >= debug.auto_dump_freq){
//		uint32_t t32; uint16_t t16;
		debug.auto_dump_cnt = 0;
		// just checking an arithm function
/*
		t16 = ign.adv_table[0];
		t16 <<= 8;
		t16 += ign.adv_table[1];
		t32 = t16;
		t32 <<= 16;
		t16= ign.adv_table[2];
		t16 <<= 8;
		t16 += ign.adv_table[3];
		t32 += t16;
		benchstats.timeoffset = div4096(t32);
*/
		benchstats_sendsnapshot();
		return;
	}
	
	if(debug.auto_dump_freq == 0xFF){	// one shot
		benchstats_sendsnapshot();
		debug.auto_dump_freq = 0;
	}
}


/* with an exactly timed loop we will measure how much of the total time
 * is consumed by the eventqueue interrupts
 */
uint16_t benchmark_idle(uint16_t cycles)
{
	uint16_t i, t, sum;
	sum = 0;
	for(i=cycles; i; i--){
		for(t=0xFFFF; t; t--){
			sum += t; // gcc please don't optimize this
		}
	}
	return sum;
}

/* consolidated using the private attribute in the 
command (see pushfuncpar)
*/
uint8_t tmp_comm(uint8_t i)
{
	uint16_t	t;
	uint8_t	ind;
	if(i==0) return '\r';
	if(i==1) return '\n';
	i-=2;
	ind=i>>3;
	t = ((uint16_t *)cmd_queue[cmd_queue_head].private)[ind]; // each word sent in 8bytes
	i &= 7;
	switch(i){	// 2 hexabytes and 2 spaces
		case 0: return ' ';
		case 1: return 'T';	// hex_hi(ind);
		case 2: return hex_lo(ind);
		case 3: return '=';
		case 4: return hex_hi(t>>8);
		case 5: return hex_lo(t>>8);
		case 6: return hex_hi(t&0x00FF);
		case 7: return hex_lo(t&0x00FF);
		default: return 'e';
	}
	return 'Q';
}
/*@}*/

