/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Michael Kristensen, mik@caffrey.dk]          *
***********************************************************
*  -- History --                                          *
*    release 0.1 - 06-03-2003                             *
*            http://caffrey.dk/megasquirt                 *
*    initial Motorola version by Bowling & Grippo (2002)  *
*            http://www.bgsoflex.com/megasquirt.html      *
\*********************************************************/


#ifndef COMM_H
#define COMM_H

#include <inttypes.h>
#include <avr/io.h>
#define CMD_QUEUE_SIZE 32


#define MT_PAGE_IDX_BOOST (2)
#define MT_PAGE_IDX_VE (4)
#define MT_PAGE_IDX_LAMBDA (5)
#define MT_PAGE_IDX_IGNADV (6)
// LAMBDAVIEW is same data as LAMBDA, but 1/x style for retarded tuningprograms
#define MT_PAGE_IDX_LAMBDAVIEW (7)
#define MT_PAGE_IDX_DUMMY (0)

#define MT_PAGE_SIZE_DUMMY (0)

// a structure to hold the private characters so they can also 
// be used as a private word
struct privch_t {
  uint8_t a;
  uint8_t b;
};

struct cmd_queue_t {
  uint8_t (*func)(uint8_t i);
  uint8_t len;    // number of bytes to send
  uint8_t count;  // number of bytes already sent
  union {
    uint8_t * private;
    uint8_t (*privatefunc)(uint8_t i);
  };
  union {
    struct privch_t privch;
    uint16_t privw;
  };
  uint8_t seq;   // sequence number used in sipr
};


// We could enumerate instead, but ansi-C promotes enumerations to
// integers (16 bit), no need for that!
#define CMD    0
#define OFFSET 1
#define VALUE  2
extern uint8_t comm_state;

extern uint8_t cmd_queue_head, cmd_queue_tail;
extern struct cmd_queue_t cmd_queue[CMD_QUEUE_SIZE];

#define comm_buffer_full() (cmd_queue_head == ((cmd_queue_tail + 1) % CMD_QUEUE_SIZE))

#if ((defined WBO2) || (defined KNOCK_8101))
#define comm_TX_plain_SIZE (64)
extern uint8_t comm_TX_plainbytes[comm_TX_plain_SIZE];
#endif


void initUART(void);
void comm(uint8_t data);

void MTstoreConfigVar(uint8_t addr, uint8_t value);

#define MT_RTVAR_COUNT (38)
uint8_t MTsendRTvar(uint8_t i);
uint8_t MTsendConfigVar(uint8_t i);
uint8_t MTtestComm(uint8_t i);
uint8_t MTcodeVer(uint8_t i);

void change_page(uint8_t page);
uint8_t send_page(uint8_t i);
void store_page(uint8_t addr, uint8_t value);
uint8_t sendConfigVar(uint8_t i);
uint8_t ConfigDump(uint8_t cnt);
uint8_t TableDump(uint8_t cnt);
uint8_t pushfunc(uint8_t (*f)(uint8_t i), uint8_t len);
uint8_t pushfuncpar(uint8_t (*f)(uint8_t i), uint8_t len, const uint8_t *);
uint8_t pushfuncpar2(uint8_t (*f)(uint8_t i), uint8_t len, const uint8_t * par, uint8_t p2);
uint8_t pushfuncsipr(uint8_t (*f)(uint8_t i), uint8_t len, uint8_t seq);

uint8_t sendVE(uint8_t i);
uint8_t sendVEfraction(uint8_t i);
uint8_t sendVEref(uint8_t i);
uint8_t srl_direct(uint8_t);
uint8_t srl_generic(uint8_t);
uint8_t srl_generic_P(uint8_t addr);
uint8_t srl_generic_P_upper(uint8_t addr);
void storeVE(uint8_t addr, uint8_t value);
void storeVEref(uint8_t addr, uint8_t value);
uint8_t sendSignature(uint8_t i);

#ifdef MSNS
void MSNSstoreConfigVar(uint8_t addr, uint8_t value);
uint8_t MSNSsendConfigVar(uint8_t addr);
#endif

/* user defined baudrate */
static inline void change_baudrate( uint8_t newbaud ){
	UBRR0H = 0;
	UBRR0L = newbaud; 
}

#endif
