/*! \file doxygen.h
    \brief Doxygen Specific file.
    
    Details.
*/

/*!
 * \mainpage VEMS - The Versatile Engine Management System
 *
 * \section intro Introduction
 *
 * This document is intended as a programmer's reference to the
 * The Versatile Engine Management Systems engine control software.
 *
 * Versatile Engine Management Systems (VEMS) is about low cost,
 * reliable, and fun do-it-yourself (DIY) engine performance.
 * The Versatile Engine Management Systems is an electronic fuel
 * injection (EFI) system based on the Atmel AVR Mega128 processor.
 * The VEMS code was originally based on the  highly respected MegaSquirt,
 * which was the first affordable opensource fuel injection system.
 * Whereas the MegaSquirt was written in assembly language,
 * the VEMS code consists of several thousand lines of modularized "C"code
 * that makes it possible to realize high quality, microcontroller based
 * realtime control applications.
 *
 * Version 1 of this code was originally based on the following contributions:
 *
 * - The fuel calculations and eeprom storage, and lots of helperl scripts
 *     were written by Michael Kristensen.
 * - The BootLoader mostly by Dana Scott.
 * - The LCD, EventQueue and the MenuSystem mostly by Marcel lGal.
 * - WBO2 by Marcell Gal, Michael Kristensen, and DaveBrul
 * - The overlap is huge, Mik had a tremendous debug/audit contribution to all parts.
 *
 * The VEMS code is not intended to be portable, but
 * crafted to fit into a special hardware based on an ATmega 128
 * CPU. A reference design of this hardware is freely available
 * as an Open Hardware Project and is available via SourceForge.
 *
 * \subsection contact Contact Information
 * If you would like to contact us regarding  VEMS, please visit
 * http://www.vems-group.org/
 *
 * VEMS electronic fuel kits as well as assembled and tested boards
 * may be ordered from
 * http://--------------------
 *
 * \section whatsnew Changes since version 2.2
 *
 * - analog filters for common sensors (rev:MarcellGal, )
 * - 2 channels: "standard" optocoupled "coil" trigger input (rev:MarcellGal, )
 * - 2 x OPA input circuits usable as VRsensor or generic amplifier (eg.EGT)
 * - 2 channels: digitally controlled WBO with currentdrive pump (YEEEAAH!) (rev:MarcellGal, )
 * - 8 smart power drivers primarily for ignition banks (high voltage friendly trace clearance), but suitable for injector/other purpose (not HWpwm-ed)
 * - 8 power drivers primarily for injector banks (each is HW-pwm capable). Suitable for other purposes, even ignition with some paint (for insulation beyond what solderprotect provides) (rev:MarcellGal, )
 * - 2 extra channels of power drivers (from logic level FETs, similar to the WBO heater FETs) (rev:MarcellGal, )
 * - 8 medium current drivers (max350mA low-side, eg.: LEDs, relays, small solenoids, MSD module)
 * - 4 channel (x1A) bidirectional stepper driver ( idle-air stepper, whatever...)
 * - low cost extra 8ch,12 bit ADC (MCP3208) for future enhancements
 * - 2 differential input, low-pass filtered analog amplifiers (eg: more EGT channels)
 * - Dallas [1-wire] (usable for whole range of apps: authentication, temp, logging-mem, USB, inter-board comm etc...)
 * - 2*RS232 port
 * - onboard 16Mbit flash memory (primarily for logging, see LoggerIntegration), or option to connect MMC card (multimegabyte)
 *
 *
 * \section copyright GENBOARD PUBLIC LICENSE
The use and distribution of this software is covered under the following <a href="doc/copyright.html">Public License.</a>

 * \section fwoverview FIRMWARE ENVIRONMENT
<center>
<img align=center src="doc/FirmwareOverView.png"
     alt="Firmware Overview"><br>
</center><p>

 */

/*!
 * \defgroup g_main_loop MAIN CYCLE LOOP
 *
 * \brief MegaSquirt AVR MAIN CYCLE LOOP.
 *
 * MegaSquirt AVR contains a small real time executive aimed at 8-bit CPUs.
 * It supports the following features:
 *    - Interrupt driven background processes
 *    - Menu System for run-time modification of parameters
 *    - Event Queue for Dynamic memory management
 *    - Asynchronous timers
 *    - Serial port character I/O
 *    - Parallel port LCD I/O
 *    - Parallel PS2 Keyboard I/O
 *    - 8 Channel A/D I/O
 *
 *
 */


  /*!
   * \defgroup g_init Initialization
   * \ingroup g_main_loop
   *
   * \brief Power On Initialization Functions.
   *
   * details ...
   */

/*!
 * \defgroup g_timing Ignition & Fuel Timing
 *
 * \brief  Ignition timing processing.
 *
 * details ...
 */


   /*!
   * \defgroup g_ign_timing Ignition Timing
   * \ingroup g_timing
   *
   * \brief Ignition Timing Functions.
   *
   * details ...
   */

       /*!
       * \defgroup g_edis_timing EDIS Ignition Timing
       * \ingroup g_ign_timing
       *
       * \brief EDIS (Electronic Distributorless Ignition System) Functions.
       *
       * details ...
       */


   /*!
   * \defgroup g_engine_timing Engine Timing
   * \ingroup g_timing
   *
   * \brief Engine Timing Functions.
   *
   * details ...
   */

       /*!
       * \defgroup g_event_queues Event Queues
       * \ingroup g_engine_timing
       *
       * \brief Event Queues allow scheduling of events precisely with little overhead.
       *
       * details ...
       */

       /*!
       * \defgroup g_heap Memory Heap Routines
       * \ingroup g_engine_timing
       *
       * \brief Routines for adding and removing items from tree-structured memory.
       *
       * details ...
       */

   /*!
   * \defgroup g_fuel Fuel Control Routines
   * \ingroup g_timing
   *
   * \brief Engine Timing Functions.
   *
   * details ...
   */


       /*!
       * \defgroup g_fuel_calc Fuel Calculation Routines
       * \ingroup g_fuel
       *
       * \brief Calculate Required Fuel.
       *
       * details ...
       */


       /*!
       * \defgroup g_fuel_control Fuel Injector Control Routines
       * \ingroup g_fuel
       *
       * \brief Fuel Injector Control Routines.
       *
       * details ...
       */
/*!
 * \defgroup g_io_control Inputs and Outputs
 *
 * \brief Control for Inputs and Outputs.
 *
 * details ...
 */

   /*!
   * \defgroup g_outputs General Purpose Outputs
   * \ingroup g_io_control
   *
   * \brief Control for General Purpose Outputs.
   *
   * details ...
   */

   /*!
   * \defgroup g_inputs General Purpose Inputs
   * \ingroup g_io_control
   *
   * \brief Control for General Purpose Inputs.
   *
   * details ...
   */

   /*!
   * \defgroup g_ad_convert Analog to Digital Conversion
   * \ingroup g_io_control
   *
   * \brief Control for Analog to Digital Conversion.
   *
   * details ...
   */

   /*!
   * \defgroup g_lcd LCD Control Routines
   * \ingroup g_io_control
   *
   * \brief Control for LCD Display.
   *
   * details ...
   */

   /*!
   * \defgroup g_keyboard Keyboard Control Routines
   * \ingroup g_io_control
   *
   * \brief Keyboard Inputs for Menu System.
   *
   * details ...
   */

   /*!
   * \defgroup g_idle_air Idle Air Control Routines
   * \ingroup g_io_control
   *
   * \brief Control Idle Air via On/Off, PWM, Stepper.
   *
   * details ...
   */

/*!
 * \defgroup g_gen_purpose General Purpose Routines
 *
 * \brief General Purpose Routines used by other Modules.
 *
 * details ...
 */

   /*!
   * \defgroup g_math General Purpose Math Routines
   * \ingroup g_gen_purpose
   *
   * \brief General Purpose Math Routines.
   *
   * details ...
   */

   /*!
   * \defgroup g_helpers Specialized Math Routines
   * \ingroup g_gen_purpose
   *
   * \brief Specialized Math Routines.
   *
   * details ...
   */

   /*!
   * \defgroup g_pwm Pulse Width Modulation Routines
   * \ingroup g_gen_purpose
   *
   * \brief Pulse Width Modulation Routines.
   *
   * details ...
   */

   /*!
   * \defgroup g_storage EEPROM Memory Storage Routines
   * \ingroup g_gen_purpose
   *
   * \brief EEPROM Memory Storage Routines.
   *
   * details ...
   */

/*!
 * \defgroup g_communications Communication Routines
 *
 * \brief Communication Routines Routines for LCD, RS-232, etc.
 *
 * details ...
 */

   /*!
   * \defgroup g_debug Debug Routines
   * \ingroup g_communications
   *
   * \brief Debug Routines.
   *
   * details ...
   */


   /*!
   * \defgroup g_menu Menu Routines
   * \ingroup g_communications
   *
   * \brief Menu Routines.
   *
   * details ...
   */


   /*!
   * \defgroup g_serial_comm Serial Communication Routines
   * \ingroup g_communications
   *
   * \brief Serial Communication Routines.
   *
   * details ...
   */


/*!
 * \defgroup g_sensors Sensor Routines
 *
 * \brief Sensor Processing Routines for NBO2, WBO2, etc.
 *
 * details ...
 */

   /*!
   * \defgroup g_ego_nbo2 Narrow Band O2 Sensor Routines
   * \ingroup g_sensors
   *
   * \brief Narrow Band O2 Sensor Routines.
   *
   * details ...
   */


   /*!
   * \defgroup g_ego_wbo2 Wide Band O2 Sensor Routines
   * \ingroup g_sensors
   *
   * \brief Wide Band O2 Sensor Routines.
   *
   * details ...
   */

   /*!
   * \defgroup g_knock Knock Sensor Routines
   * \ingroup g_sensors
   *
   * \brief Knock Sensor Routines.
   *
   * details ...
   */


