/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Michael Kristensen, mik@caffrey.dk]          *
***********************************************************
*  -- History --                                          *
*	 release 0.2 - 25-07-2003  cell at x-dsl.hu			  *
*			 inj moved to eventqueue, OCR0 become 1kHz	  *
*    release 0.1 - 06-03-2003                             *
*            http://caffrey.dk/megasquirt                 *
*    initial Motorola version by Bowling & Grippo (2002)  *
*            http://www.bgsoflex.com/megasquirt.html      *
\*********************************************************/
   /*!
   * \addtogroup g_fuel_control Fuel Injector Control Routines
   */
/*@{*/

#include "global.h"
#include "actuators.h"
#include "helpers.h"
#include "adc.h"

#include "fuelcontrol.h"
#include "fuelcalc.h"
#include "comm.h"
#include "iac.h"
#include "eventqueue.h"
#include "dispatcher.h"
#include "injconf.h"
#include "ignconf.h"
#include "edis.h"
#include "benchmark.h"
#include "timing.h"
#include "knock.h"

#ifdef CAPTURE_ALIEN_ECU
#include "ign_logging.h"
#endif

#include <inttypes.h>
#include <avr/io.h>
#include <stdlib.h>
#include <avr/interrupt.h>

#include "lbus.c"
#include "injector.c"
#include "arithm.c"

#include <avr/io.h>
#include <inttypes.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include <avr/sfr_defs.h>

struct squirt_t inj_port[INJECTORS];

volatile struct engine_t engine;
extern uint8_t crank_timeout;

#ifdef GENBOARDv3

// moved to injector.c

#else

uint8_t sled_cnt;  // only using this for turning on/off the LED_SQUIRT

// only genboard v2.2
// TODO: use this below
static inline void disable_inj_pwm(void)
{
	TCCR1A &= ~( _BV(COM1A1) | _BV(COM1A0) );	// disable pwm;
	// this is only needed at pwming:
	PORTG &= ~_BV(FLYBACK_A);                 // flyback disable
}

__inline__ void stopinj1(void)
{
//      inj_port1.status &= ~(_BV(firing) | _BV(enabled) | _BV(scheduled));
      TCCR1A &= ~( _BV(COM1A1) | _BV(COM1A0) );	// disable pwm
#ifdef FETDRIVER_NONINVERTING
      PORTB &= ~_BV(PWM_A);                      // inject1 deactive
#else
      PORTB |= _BV(PWM_A);                      // inject1 deactive
#endif
      PORTG &= ~_BV(FLYBACK_A);                 // flyback disable
      sled_cnt &= ~0x01;
      if (!sled_cnt) ledstate &= ~_BV(LED_SQUIRT);  // sled off
#ifdef FRIED_FET_BUT_RACE_HACK
	PORTB &= ~_BV(CLT_FAN); // start injector driven by lowside PNP
#endif
}

__inline__ void stopinj2(void)
{     
//	  inj_port2.status &= ~(_BV(firing) | _BV(enabled) | _BV(scheduled));
      TCCR1A &= ~( _BV(COM1B1) | _BV(COM1B0) );	// disable pwm
#ifdef FETDRIVER_NONINVERTING
	  PORTB &= ~_BV(PWM_B);                      // inject2 deactive
#else
      PORTB |= _BV(PWM_B);                      // inject2 deactive
#endif
      PORTG &= ~_BV(FLYBACK_B);                 // flyback disable
      sled_cnt &= ~0x02;
      if (!sled_cnt) ledstate &= ~_BV(LED_SQUIRT);  // sled off
}

// Should these disable PWM-ing ? 
// pwming only needs to be stopped, when injection was stopped.
// if inj duty >= 100%, why would we want to stress the injector with current
// when it is already open? The continuous PWM is the right behaviour in that case.
// when inj duty < 100% the stopinjx() will take care of 
// disable_inj_pwm()...
__inline__ void startinj1(void)
{
    ledstate |= _BV(LED_SQUIRT);  // sled on
    sled_cnt |= 0x01;
#ifdef FETDRIVER_NONINVERTING
	PORTB |= _BV(PWM_A);               // inject1 deactive
#else
    PORTB &= ~_BV(PWM_A);      // inject1 active
#endif
#ifdef FRIED_FET_BUT_RACE_HACK
	PORTB |= _BV(CLT_FAN);	// start injector driven by lowside PNP
#endif
}

__inline__ void startinj2(void)
{
    ledstate |= _BV(LED_SQUIRT);  // sled on
    sled_cnt |= 0x02;
#ifdef FETDRIVER_NONINVERTING
    PORTB |= _BV(PWM_B);      // inject2 active
#else
	PORTB &= ~_BV(PWM_B);      // inject2 active
#endif
}

__inline__ void injAgopwm(void)
{
	OCR1A = inj_port[0].pwm_dc * 10;  // pwm duty cycle, multiply by 10 to increase range
    TCCR1A |= _BV(COM1A1) | _BV(COM1A0);	// enable pwm
	PORTG |= _BV(FLYBACK_A);   // flyback enable
}

__inline__ void injBgopwm(void)
{
	OCR1B = inj_port[1].pwm_dc * 10;  // pwm duty cycle, multiply by 10 to increase range
    TCCR1A |= _BV(COM1B1) | _BV(COM1B0);	// enable pwm
	PORTG |= _BV(FLYBACK_B);   // flyback enable
}

#endif

/* 
 * only call from interrupt (between cli-sei)
 */
__inline__ void fuelpump_on(void)
{
#ifdef GENBOARDv3
	digitalout(config.fuelpump_channel, DHIGH);
#else
	PORTB |= _BV(FUELPUMP);
#endif
}

/*
 * only call from interrupt (between cli-sei)
 */
__inline__ void fuelpump_off(void)
{
#ifdef GENBOARDv3
	digitalout(config.fuelpump_channel, DLOW);
#else
	PORTB &= ~_BV(FUELPUMP);
#endif
}

/*
 * Only call from userspace
 */
__inline__ void engine_off(void)
{
	engine.status = 0; // engine is stalled, clear all in engine
    engine.status_ext |= _BV(o2_not_ready) | _BV(crank_enable);
    engine.status_trig = 0;
	
	// the toothwheel is initially not in sync
	etime.tooth_cnt = 255;

    etime.cam_sync_cnt = 0;

	engine.rpm_period = 0xFFFF;
	engine.rpm = 0;
	engine.rpm_vhr = 0;
#ifndef NOIGN
	// TODO: remove, since useless. See table_lookup()
	engine.ignadv = config.ign_crank_advance;
#ifdef KNOCK_8101
	reset_knock_adjust();
#endif	// KNOCK_8101
#endif	// NOIGN
	crank_timeout = CRANK_TIMEOUT;

	cli();
    fuelpump_off(); //fuelpump off
	sei_nop_cli();
// the idlevalve shouldn't be controlled from here, but in iac.c instead
#ifdef GENBOARDv3
    digitalout(config.iac_sol_channel, DLOW);
#else
	// this might switch off some other v2 hacks,
	// like WOT_OUTPUT_TPS_THRESHOLD (shouldn't be a problem)
	// check idle_solenoid()
    PORTB &= ~_BV(IDLEVALVE); // idlevalve off
#endif
	sei_nop_cli();

// ledstate is the same (cache) for v3. The HWout will be different 
    ledstate &= ~( _BV(LED_SQUIRT) | _BV(LED_WARMUP) );   //sled off, wled off

	sei();
	
	fill_all_squirters(0);
}


__inline__ void engine_is_running(void) {
	fuelpump_on();
	engine.status |= _BV(running);
}


__inline__ void injector_pwm_init(void) {
#ifdef GENBOARDv3

	// setup OC0 for 16usec (prescaler=1), PWM mode
	// inverted mode if defined by FETDRIVER_INVERTING
	// TODO: should we configure the same way and invert at
	// OCR0 assignment like:	OCR0=255- ... ?
	// Any issues at min (0) and max settings (who cares?)

	// TCCR0 = could be OK too...
	TCCR0 |= (_BV(WGM00)| _BV(WGM01) | _BV(CS00) | _BV(COM01) 
#ifdef FETDRIVER_INVERTING
	| _BV(COM00)
#endif
	);
#else

	// init pwm
	// Set OCnA/OCnB/OCnC on compare match, clear OCnA/OCnB/OCnC at TOP
	// fast pwm mode, TOP is stored in ICR1
	TCCR1A = _BV(WGM11);
	TCCR1B = _BV(WGM13) | _BV(WGM12) | _BV(CS10);
	ICR1 = PWM_FREQUENCY; // change pwm frequency

#endif

}


// #include "eventqueue.c"
#include "reenable.c"
/* INTERRUPT: Timer @ 0.1 ms,  with EVENTQUEUE @ 1 ms 			   	*/ 
/* OBSOLETE. We have a handler2k, so this can be used for */
/* anything ...		*/
SIGNAL (SIG_OUTPUT_COMPARE0) {
#ifdef GENBOARDv3
#warning "OCR0 selfstimuli broken!"
#else
	uint32_t capturedtime;
	uint16_t t;
	startinj2();
	t = debug.forced_pw << 4;
	// readTMR2(); implicite in the relative event_insert...:
	event_insert_rel(t, INJBSTOP);
	inj_port[1].stopat = etime.now + t;
	capturedtime = now32();
	engine.rpm_period = capturedtime - etime.ptrig_capturedtime_prev;
#ifdef BENCHMARK
	if (engine.rpm_period < benchstats.lowest_rpm_p)
		benchstats.lowest_rpm_p = engine.rpm_period;
	if (engine.rpm_period > benchstats.highest_rpm_p)
		benchstats.highest_rpm_p = engine.rpm_period;
#endif
	etime.ptrig_capturedtime_prev = capturedtime;
	engine.status_ext |= _BV(new_rpm);  // start a rpm calculation

#ifdef BENCHMARK
	benchstats.ocr0cnt++;
#endif
	alarm2core();
#endif
}


uint32_t spark_time;
uint16_t spark_rpm;

#include "arithm.c"

// calculate alien ignition advance from measured data
void calc_ign_advance(void) {
	uint16_t diff;

	new_spark=0;

	if (spark_time >> 16) {
#ifdef BENCHMARK
		benchstats.ign_adv_err++;
#endif
		engine.alien_ignadv = 0; // we don't know the advance
	} else {
		uint16_t ia;	// ignition advance
		// reference input:
		// (spark_rpm==6000) RPM == 100/sec == 10ms == (spark_time==2500 [4usec])
		// result == 360 crankdeg == 720 [0.5crankdeg]
		// note that it is 6* higher ignadv than possible, still no overflow
		diff = mult16_16(spark_rpm, spark_time) >> 8;	// for ref-input: 58594
	
		// TDC is ign_tdcdelay deg after the missing tooth [0.5 crankdegree]
		ia = config.ign_tdcdelay;
	
		// we want [0.5crankdeg] units, therefore the magic constant:
		ia -= mult16_16(805, diff) >> 16;
		if((ia>>8) != 0)	// too high value, railing
			engine.alien_ignadv = 0xFF;
		else
			engine.alien_ignadv = (uint8_t)ia;
#ifdef CAPTURE_ALIEN_ECU
		if(((debug.enable_flags & _BV(5))) // don't enable mde.. bit0 at the same time !
		   && (*comm_TX_plainbytes == 0)	// the signalling byte is already zeroed by comm.c (gcc-at-powerup)
		   && ! comm_buffer_full()){
			alien_detailed_log();
		}
#endif
	}
}

/*@}*/

