/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Michael Kristensen, mik@caffrey.dk]          *
***********************************************************
*  -- History --                                          *
*    release 0.1 - 06-03-2003                             *
*            http://caffrey.dk/megasquirt                 *
*    initial Motorola version by Bowling & Grippo (2002)  *
*            http://www.bgsoflex.com/megasquirt.html      *
\*********************************************************/

/*
  Generally all functions in this file should be optimized - 
  that is, hand coded avr-assembler.
  
  Especially the division routines should be optimized, since the avr is
  lacking a div-instruction. Check Atmel application note for at fast 
  assembler-division.

  Mik: division need not be optimized. It should be eliminated.
  Note that 1/(x2-x1) can be precalculated for the table rpm and
  map values, so it becomes multiplication.
*/

   /*!
   * \addtogroup g_helpers Specialized Math Routines
   */
/*@{*/

#include "global.h"
#include "helpers.h"
#include "ve.h"
#include <inttypes.h>
#include <stdlib.h>

#include "arithm.c"

// 65535/x table generated and model verified with the help of
// (JTune CVS) org.vemsgroup.firmware.datatypes.Linterp.main();
uint16_t PROGMEM RX21LOOKUP[] = {
65535,65535,32768,21845,16384,13107,10923,9362,8192,7282,6554,5958,5461,5041,4681,4369
,4096,3855,3641,3449,3277,3121,2979,2849,2731,2621,2521,2427,2341,2260,2185,2114
,2048,1986,1928,1872,1820,1771,1725,1680,1638,1598,1560,1524,1489,1456,1425,1394
,1365,1337,1311,1285,1260,1237,1214,1192,1170,1150,1130,1111,1092,1074,1057,1040
,1024,1008,993,978,964,950,936,923,910,898,886,874,862,851,840,830
,819,809,799,790,780,771,762,753,745,736,728,720,712,705,697,690
,683,676,669,662,655,649,643,636,630,624,618,612,607,601,596,590
,585,580,575,570,565,560,555,551,546,542,537,533,529,524,520,516
,512,508,504,500,496,493,489,485,482,478,475,471,468,465,462,458
,455,452,449,446,443,440,437,434,431,428,426,423,420,417,415,412
,410,407,405,402,400,397,395,392,390,388,386,383,381,379,377,374
,372,370,368,366,364,362,360,358,356,354,352,350,349,347,345,343
,341,340,338,336,334,333,331,329,328,326,324,323,321,320,318,317
,315,314,312,311,309,308,306,305,303,302,301,299,298,297,295,294
,293,291,290,289,287,286,285,284,282,281,280,279,278,277,275,274
,273,272,271,270,269,267,266,265,264,263,262,261,260,259,258,257
};

/*
 * downscales val: return (val * scale / 256)
 * cheaper than scaleprec_t scale and good result
 */
static inline uint8_t scaledown_val(uint8_t val, scale_t scale)
{
	uint16_t t16 = val * scale;
	return t16>>8;
}
/*
 * return (val * scale / 0x1000)
 * 16 bit version
 */
static inline uint16_t scaledown_precval(uint16_t val, scaleprec_t scale)
{
	uint32_t t32 = mult16_16(scale, val);
	return t32 >> 16;
}
/***************************************************************************/
/*** FAST linear interpolation                                           ***/
/*** x1 <= x2 assumed		(y1, y2 can be any)			 ***/
/***************************************************************************/
uint8_t linear_interp(uint8_t x1, uint8_t x2, uint8_t y1, uint8_t y2, uint8_t x)
{
  	uint8_t x21, dx;
        uint16_t rx21;
        scale_t scale;

        if (x <= x1) // below/equal lower bound
            return y1;

        if (x >= x2) // above/equal upper bound
            return y2;

        x21 = x2 - x1;
        // (x2-x1)!=0 because of the previous checks .. but it wouldn't cause problem

        dx = x - x1;
        rx21 = div_65535_x( x21 );

        // rx21 * dx <= 65535 by nature. But only upper 8 bit counts
        scale = mult16_8(rx21, dx) >> 8;

        if (y2 <= y1) {
            // negative slope
            // return y1 - ((y1-y2)*scale) ;
            uint8_t dy = y1 - y2;
            return y1 - scaledown_val(dy, scale);
        } else {
            // positive slope
            // return y1 + ((y2-y1)*scale) ;
            uint8_t dy = y2 - y1;
            return y1 + scaledown_val(dy, scale);
        }
}


/***************************************************************************/
/*** linear interpolation, slightly more expensive than linear_interp()  ***/
/*** since x1 <= x2 NOT assumed		(and y1, y2 can be any)				 ***/
/***************************************************************************/
uint8_t linear_interp_allow_swap(uint8_t x1, uint8_t x2, uint8_t y1, uint8_t y2, uint8_t x)
{
	if( x2 >= x1){
		return linear_interp( x1, x2, y1, y2, x);
	} else {
		return linear_interp( x2, x1, y2, y1, x);
	}
}


/************************************************/
/* x=8, y=16 bit version of the above		    */
/* x1 <= x2 assumed		(y1, y2 can be any) */
/************************************************/
uint16_t linear_interp16(uint8_t x1, uint8_t x2, uint16_t y1, uint16_t y2, uint8_t x)
{
  	uint8_t x21, dx;
        uint16_t rx21;
        scaleprec_t scale;

        if (x <= x1) // below/equal lower bound
            return y1;

        if (x >= x2) // above/equal upper bound
            return y2;

        x21 = x2 - x1;
        // (x2-x1)!=0 because of the previous checks .. but it wouldn't cause problem

        dx = x - x1;
        rx21 = div_65535_x( x21 );

        // rx21 * dx <= 65535 by nature
        scale = mult16_8(rx21, dx);

        if (y2 <= y1) {
            // negative slope
            // return y1 - ((y1-y2)*scale) ;
            uint16_t dy = y1 - y2;
            return y1 - scaledown_precval(dy, scale);
        } else {
            // positive slope
            // return y1 + ((y2-y1)*scale) ;
            uint16_t dy = y2 - y1;
            return y1 + scaledown_precval(dy, scale);
        }
}

/*****************************************************
 * half of linear_interp() function
 * almost like y1=0 y2=255
 */
uint8_t linear_interp_fixed_y(uint8_t x1, uint8_t x2, uint8_t x)
{
	uint8_t x21, dx;
	uint16_t rx21;
	if (x <= x1) return 0;
	if (x >= x2) return 0xff;

	x21 = x2 - x1;
	rx21 = div_65535_x( x21 );

	dx = x - x1;
	return mult16_8( rx21, dx ) >> 8;
}

/***************************************************************************/
/*** search table                                                        ***/
/***************************************************************************/
/*
Search a table for the location of "item".

returns:
  tbl[lindex] <= item < tbl[uindex]
  in general: lindex = uindex - 1
  except when (uindex == 0): lindex = uindex = 0

  if "item" is larger than all elements in the table, the max position
  will be returned
  I propose to change this: With 8 long table (0..7) to return
  min 0,1 even if item < tbl[0]
  max 6,7 even if item > tbl[7]
*/

// the array to be searched must be increasing (or equal)
void search_table(uint8_t *tbl, uint8_t tbl_length, uint8_t item, struct search_table_t *r) {
  uint8_t i, searching, index;

  searching = 1;
  index = 0;

  for (i=0; (i<tbl_length) && searching; i++) {
    index = i;
    if (item < tbl[i]) {
      searching = 0;
    }
  }

  if (index == 0) {
    r->lindex = r->uindex = 0;
  } else {
    r->lindex = index-1;
    r->uindex = index;
  }
}


/***************************************************************************/
/*** mult div100                                                         ***/
/***************************************************************************/
uint16_t mult_div100(uint8_t a, uint16_t b) {
// NOTE that first parameter is 8 bit (correction factor around 100), 
// x*y/100 == appr.  x*y* 41 / 4096 (error in 0.1%)
	uint16_t t;
	uint32_t t32;	// let's be sure that the bitlength is what we think...
	t = 41*a;	// do not put into one expression with previous
	t32 = mult16_16(t, b);
	return div4096(t32);
}

// x*y/10 = appr. x*y* 205 / 2048 (error in 0.1%)
uint16_t mult_div10(uint8_t a, uint16_t b) {
	uint16_t t;
	uint32_t t32;	// let's be sure that the bitlength is what we think...
	t = 205*a;	// do not put into one expression with previous
	t32 = mult16_16(t, b);
	return t32 >> 11;
}

/* 
 * return a*b/10000 (a*b*41*41/4096/4096) (error in 0.2%)
 */
uint16_t mult_div10k(uint16_t a, uint16_t b) {
	uint32_t t32;
	uint16_t t16, t17;
	// always check the assembly output if you change a bit
	t16 = mult16_8(a, 41) >>8;
	t17 = mult16_8(b, 41) >>8;
	t32 = mult16_16(t16, t17);
	//t32 *= (uint32_t)(41*41);// unfortunately this is a long multiply, but somehow gcc overoptimizes (return 0) if I try to fit in 16 bit
	return t32>>8;
}

/* 
 * return a*b/596 (a*b*128*220/4096/4096)
 */
uint16_t mult_div596(uint16_t a, uint16_t b) {
	uint32_t t32;
	uint16_t t16, t17;
	// always check the assembly output if you change a bit
	t16 = mult16_8(a, 128) >>8;
	t17 = mult16_8(b, 220) >>8;
	t32 = mult16_16(t16, t17);
	return t32>>8;
}

// div by 25575 (-1/1000) but does not lose precision
uint16_t mult_div25600(uint16_t a, uint16_t b) {
	uint32_t t32;
	uint16_t t16, t17;
	// always check the assembly output if you change a bit
	t16 = mult16_8(a, 16) >>8;
	t17 = mult16_8(b, 41) >>8;
	t32 = mult16_16(t16, t17);
	//t32 *= (uint32_t)(41*41);// unfortunately this is a long multiply, but somehow gcc overoptimizes (return 0) if I try to fit in 16 bit
	return t32>>8;
}
/***************************************************************************/
/*** median filter                                                       ***/
/***************************************************************************/
// this is hardcoded for a three byte buffer
uint8_t median_filter(struct mfilter_t *f, uint8_t in) {
  uint8_t m1, m2;

  // shift all elements one place
  f->buf[2] = f->buf[1];
  f->buf[1] = f->buf[0];
  f->buf[0] = in;

  // find the largest of buf0 and buf1
  if (f->buf[0] > f->buf[1])
    m1 = f->buf[0];
  else
    m1 = f->buf[1];

  // find the largest of buf1 and buf2
  if (f->buf[1] > f->buf[2])
    m2 = f->buf[1];
  else
    m2 = f->buf[2];

  // return the smaller value
  if (m1 < m2)
    return m1;
  else
    return m2;

}


/***************************************************************************/
/*** limit                                                               ***/
/***************************************************************************/
int16_t limit_i16(int16_t val, int16_t upper, int16_t lower) {
  if (val > upper)
    return upper;
  else if (val < lower)
    return lower;
  else
    return val;
}

int8_t limit_i8(int8_t val, int8_t upper, int8_t lower) {
  if (val > upper)
    return upper;
  else if (val < lower)
    return lower;
  else
    return val;
}

uint8_t limit_u8(uint8_t val, uint8_t upper, uint8_t lower) {
  if (val > upper)
    return upper;
  else if (val < lower)
    return lower;
  else
    return val;
}

/***************************************************************************/
/*** min                                                                 ***/
/***************************************************************************/
__inline__ uint8_t min(uint8_t a, uint8_t b) {
  if (a < b)
    return a;
  else
    return b;
}

/***************************************************************************/
/*** max                                                                 ***/
/***************************************************************************/
__inline__ uint8_t max(uint8_t a, uint8_t b) {
  if (a > b)
    return a;
  else
    return b;
}


/* simple table-lookup and interpolation.
	(index / 16) is offset; (index % 16) is fraction;
	note:
	the highest bin is not interpolated if (index>>4)+1 >= table_size )
*/
uint8_t interpol16(uint8_t *tab, uint8_t table_size, uint8_t index)
{
	uint8_t th, tl;
	uint16_t t16;

	// 0..0xFF not 0..100
	th = index >> 4;
	tl = index & 0x0F;
	if( (th + 1) >= table_size ){	// last bin is used
		return tab[table_size - 1];
	} else {
		t16 = tab[th] * (16 - tl);
		t16 += tab[th + 1] * tl;
		return t16 >> 4;
	}
}

/* simple table-lookup and interpolation.
	(index / 8) is offset; (index % 8) is fraction;
	note:
	the highest bin is not interpolated if (index>>4)+1 >= table_size )
*/
uint8_t interpol8(uint8_t *tab, uint8_t table_size, uint8_t index)
{
	uint8_t th, tl;
	uint16_t t16;

	// 0..0xFF not 0..100
	th = index >> 5;
	tl = index & 0x1F;
	if( (th + 1) >= table_size ){	// last bin is used
		return tab[table_size - 1];
	} else {
		t16 = tab[th] * (32 - tl);
		t16 += tab[th + 1] * tl;
		return t16 >> 5;
	}
}

/*@}*/

