/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Marcell Gal cell at x-dsl.hu]                *
***********************************************************
* This file is included, not linked
* used to drive the 259 chips connected to LBUS
\*********************************************************/
   /*!
   * \addtogroup g_outputs General Purpose Outputs
   */
/*@{*/
#ifndef __LBUS_C__
#define __LBUS_C__

#include "lbus.h"

#ifdef GENBOARDv3
#include <inttypes.h>
#include <avr/io.h>

/* sets a bit to  on the addressed 259 chip 
	MUST be called from cli()...
	valbit bit7 is the value to send, 
	6:4 is the address	
	3:0 must be 0
*/
static inline void l259_packed(uint8_t valbit, volatile uint8_t *csport, uint8_t cs)
{
	uint8_t t;
/* preserve 3:0 signals */
	t = PORTC;
	t &= 0x0F;
	// gcc, please optimize this to a compile-time constant
	// t |= ((bit << 4) | (val ? 0x80 : 0));	// bit can be 0..7

	// if gcc is lame, the definition of val allows:
	t |= valbit;

	PORTC = t;
	// toggle the CS_... signal
	*csport &= (uint8_t)~_BV(cs);
	*csport |= _BV(cs);
}

/* sets a bit to  on the addressed 259 chip 
	MUST be called from cli()...
	val must be DHIGH(0x80) if you want the bit to be set, 0 for clear
	
	l259 can be remembered from "let 259..."
	This could call l259_packed();
*/
static inline void l259(uint8_t val, uint8_t bit, volatile uint8_t *csport, uint8_t cs)
{
	uint8_t t;
/* preserve 3:0 signals */
	t = PORTC;
	t &= 0x0F;
	// gcc, please optimize this to a compile-time constant
	// t |= ((bit << 4) | (val ? 0x80 : 0));	// bit can be 0..7

	// if gcc is lame, the definition of val allows:
	t |= ((bit << 4) | val);

	PORTC = t;
	// toggle the CS_... signal
	*csport &= (uint8_t)~_BV(cs);
	*csport |= _BV(cs);
}

// these are repeated in digitalout()
#define S259(val, bit) l259(val, bit, (uint8_t *)&CS_S259_PORT, CS_S259)
#define P259(val, bit) l259(val, bit, (uint8_t *)&CS_P259_PORT, CS_P259)
#define I259(val, bit) l259(val, bit, (uint8_t *)&CS_IGN259_PORT, CS_IGN259)


#endif	// GENBOARDv3
#endif	// __LBUS_C__

/*@}*/

