/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Marcell Gal]
***********************************************************
* INT7 degree-based phasesignal is not available on most engines.
* This is experimental. You don't need this.
* This is not literally injection anyway, this is special.
* Forget this. You have been warned.
\*********************************************************/

   /*!
   * \addtogroup g_ign_timing Ignition Timing
   */
/*@{*/

#include "fuelcontrol.h"
#include "fuelcalc.h"
#include "global.h"
#include "adc.h"
#include "lcd.h"
#include "comm.h"
#include "iac.h"
#include "eventqueue.h"
#include "phasesignal.h"
#include "edis.h"

#include <avr/io.h>
#include <inttypes.h>
#include <avr/signal.h>
#include <avr/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sfr_defs.h>

#define PHASE_RES 1440

uint8_t pwdata(uint8_t i)
{
   switch(i){
#ifdef PHASESIGNAL
	case 0: return hex_hi(pwdisplay.toothcount);
	case 1: return hex_lo(pwdisplay.toothcount);
#endif	
	case 2: return ' ';
	case 3: return hex_hi(pwdisplay.persum>>24);
	case 4: return hex_lo(pwdisplay.persum>>24);
	case 5: return hex_hi(pwdisplay.persum>>16);
	case 6: return hex_lo(pwdisplay.persum>>16);
	case 7: return hex_hi(pwdisplay.persum>>8);
	case 8: return hex_lo(pwdisplay.persum>>8);
	case 9: return hex_hi(pwdisplay.persum);
	case 10: return hex_lo(pwdisplay.persum);
	case 11: return '\r';
	case 12: return '\n';
	default: return 'E';
	}
}

#ifdef PHASESIGNAL
uint8_t phase_mode;
static uint16_t phasedegree;
static uint8_t BAold, found0, errstep; /* the old value of B, A signals */

static __inline__ void incdigitchar(uint8_t *p)
{
	*p = (*p) + 1;
	if(*p > '9')
		*p = '0';
}

#define ER 2
/* decoding array for ABZ encoder with newB,newA,oldB,oldA bits as address 
it is cleaner and faster and less flash than a serious bit-operation wizardry */
signed char increment_array[16] PROGMEM = {0, -1, 1, ER,	1, 0, ER, -1,	-1, ER, 0, 1,	ER, 1, -1, 0 };

/* this is a dummy busy-loop that only displays phasecounter on LCD, and keeps track 
of 2 phase-inputs... */
static __inline__ void phase_check(void)
{
	uint8_t t;
	signed char a;
	t = PINE & 0x38 ;
	if(t == 0) {
		phasedegree = 0;	// all 3 inputs are zero (active)
		// notify that we found index-signal
		incdigitchar(&found0);
		update_char(found0, 6);
	} else {
		t &= ~0x20; /* Z signal is irrelevant */
		t >>= 1;
		
		/* casting signed does not work right ???? */
		a = PRG_RDB(&increment_array[ BAold | t]);
		if(a == 1){
			phasedegree ++;
			if(phasedegree >= PHASE_RES) phasedegree -= PHASE_RES;
		} else if(a == -1){
			phasedegree --;
			if(phasedegree >= 0x8000) phasedegree += PHASE_RES;
		} else if(a == 2){
			incdigitchar(&errstep);
			update_char(errstep, 5);
		};
		
		BAold = t >> 2;
	}
}
void phase_check_ext(void)
{
	phase_check();
}

void phase_init(void)
{
/* TODO: INT7 should also depend on bit TACH_ON_INT6_OR_7 in config13 
//	EICRB |= _BV(ISC71); // falling edge
//	EIMSK |= _BV(INT7);  // enable interrupt
//	phase_mode = 'u' ;
*/
	// PORTE input
	DDRE = 0;
	// OCR2 turnoff
	TIMSK &= ~ _BV(OCIE2);
	// turn off interrupts
	EIMSK &= ~ (_BV(INT6) | _BV(INT7));
	update_char('K',7);
	cli();
	phasedegree = 0;
	found0 = errstep = '0';
	BAold = (PINE & 0x18) >> 3;
}

void phase_idle(void)
{
	uint16_t i;

	DDRA &= ~8;		// PINA3 input
	PORTA |= 8;		// PINA3 pullup
	_delay_loop_1(99);	// wait a little
	if((config.lcd_c0 & _BV(phasesignoff))
		|| ((PINA & 8) == 0)){	// someone pulled it low
		DDRA |= 8;	// turn PINA3 back to output
		return;
	}
	phase_init();

	while(1) {
		for(i=29999; i; i--){
			phase_check();
		}
		//cli();
		phase_check();
		int2BCDn(phasedegree, 0, BCD_DIGIT4);
		phase_check();
		// sei();
		for(i=29999; i; i--){
			phase_check();
		}
		//cli();
		phase_check();
		if (lcd.init) {
			lcd_init_seq();
		} else {
			lcd_refresh();
		}
		phase_check();
		// sei();
	}
}

/***************************************************************************/
/*** INTERRUPT: ignition (triggers on falling edge)                      ***/
/***************************************************************************/

SIGNAL(SIG_INTERRUPT7) {
	int myper; // get it from tachin
	
	myper = pwdisplay.rpm;
	phasedegree++;
	if(phasedegree==90) phasedegree=0;
//	phasesignalcnt=0;	// so that timer knows we are OK. It will stop injection otherwise

	if(phase_mode == 'u') {
		stopinj12();
		return;
	}

	if(phase_mode == 't') { // termelo mod 
	if (myper < RPM(700)){
		if (phasedegree>56) stopinj12();
		else if (phasedegree>42) startinj12();
		else stopinj12();
	} else if (myper <RPM(500)){
		if (phasedegree>55) stopinj12();
		else if (phasedegree>43) startinj12();
		else stopinj12();
	} else if (myper <RPM(300)){
		if (phasedegree>54) stopinj12();
		else if (phasedegree>45) startinj12();
		else stopinj12();
	} else stopinj12(); 
	return;
	}

	// phase_mode == 'e', cannot be anything else
	if (myper < RPM(700)){
		if (phasedegree>73) stopinj12();
		else if (phasedegree>60) startinj12();
		else stopinj12();
	} else if (myper <RPM(500)){
		if (phasedegree>72) stopinj12();
		else if (phasedegree>61) startinj12();
		else stopinj12();
	} else if (myper <RPM(300)){
		if (phasedegree>71) stopinj12();
		else if (phasedegree>62) startinj12();
		else stopinj12();
	} else stopinj12(); 
	
}

uint8_t newmode(uint8_t i)
{
	switch(i){
		case 0: return 'n';
		case 1: return 'e';
		case 2: return 'w';
		case 3: return '=';
		case 4: return phase_mode;
		case 5: return '\r';
		case 6: return '\n';
		default: return 'R';
	}
}
#endif

/*@}*/

