/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Marcell Gal cell at x-dsl.hu]                *
***********************************************************
* used for simple table-configurable port actions         *
\*********************************************************/

#ifndef __PORTACT_C__
#define __PORTACT_C__

#include "portact.h"

/* 
every line in the table contains a set of portacts, 2..3 of which
must be executed at a time (init, off, on)
Up to 8 lines are possible for an 8 cyl fully sequential.

Called from dispatcher and timing.c, and replace ignconf.c, injconf.c
and starinj, stopinj, gopwm port actions.
Note that the COND_STOP and event_insert() are outside the scope of this
function, this is just the port_action.

fromlen gives the offsets: 23 means max 3 actions executed
starting from linestart+2 (see defines in portact.h)

TODO: take inj_port (struct squirt_t) as parameter (or OCR value?)
TODO: case ...: // (\w+) are symbolic names so configuration is simpler, 
and survives changes (which is needed to organize a bit) => perl can generate 8 bit numbers

thumbrule: odd means take the port high
*/
/*!
 * \ingroup g_fuel_control
 * \brief Simple table-configurable port actions.
 *
 * \return none
 */

static inline do_portact(uint8_t *tab, uint8_t line, uint8_t fromlen )
{
	uint8_t from, len, *p;
	from = fromlen >> 4;
	len = fromlen & 0x0F;
	
	p = tab + (line<<3) + from; // tab + 8*line gives the linestart address
	do{
		switch(*p++){
		case 0x00: return; // nop, AND we don't look further
		case 0x04: // B5_low (OC1A)
			TCCR1A &= ~( _BV(COM1A1) | _BV(COM1A0) );	// disable PWM
			PORTB &= ~_BV(5);
		case 0x05: // B5_hi (OC1A)
			TCCR1A &= ~( _BV(COM1A1) | _BV(COM1A0) );	// disable PWM
			PORTB |= _BV(5);
		case 0x06: // B5_pwm (OC1A)
			// OCR1A = inj_port1.pwm_dc * 10;  // pwm duty cycle,
			TCCR1A |= _BV(COM1A1) | _BV(COM1A0);	// enable pwm
		// case 0x07:
		
		case 0x08: // B6_low (OC1B)
			TCCR1A &= ~( _BV(COM1B1) | _BV(COM1B0) ); // disable PWM
			PORTB &= ~_BV(6);
		case 0x09: // B6_hi (OC1B)
			TCCR1A &= ~( _BV(COM1B1) | _BV(COM1B0) ); // disable PWM
			PORTB |= _BV(6);
		case 0x0A: // B6_pwm (OC1B)
			// OCR1B = inj_port2.pwm_dc * 10;  // pwm duty cycle, multiply by 10 to increase range
			TCCR1A |= _BV(COM1B1) | _BV(COM1B0);	// enable pwm
		// case 0x0B:
		
		case 0x0C: // G3_low
			PORTG &= ~_BV(3);
		case 0x0D: // G3_hi
			PORTG |= _BV(3);
		case 0x0E: // G4_low
			PORTG &= ~_BV(4);
		case 0x0F: // G4_hi
			PORTF |= _BV(4);
			
			
		case 0xFF: break; // nop, BUT we look further
		}
		len--;
	} while (len);
}

#endif

