   /*!
   * \addtogroup g_helpers Specialized Math Routines
   */
/*@{*/

#ifndef __SWITCHER_C__
#define __SWITCHER_C__

#include <avr/delay.h>

/* this is an inefficient busyloop based switcher - used for experiments
*/

#ifndef SWITCHER_PORT
#define SWITCHER_PORT  PORTB
#define SWITCHER_PORT_DDR	DDRB
#define SWITCHER_PORT_DDR_VAL	 (0xE0)
// OC1A and OC1B control the gates:
#define SWITCHER_STATE1 (_BV(5))
#define SWITCHER_STATE2 (_BV(6))
// both are off (remember, FETdrivers invert!)
#define SWITCHER_STATE0 (_BV(5) | _BV(6))
// 20 usec with the port setting
#define SWITCHER_DELAY (106)
// how many cycles to go through before bailout: 25 = 1 sec appr.
#define SWITCHER_CYCLES (1000)

#endif

static inline void switcher_init(void)
{
	SWITCHER_PORT_DDR = SWITCHER_PORT_DDR_VAL;
	SWITCHER_PORT = SWITCHER_STATE0;
}

/* TODO: test efficiency in function of frequency, and varying the time between
switching 1 on and 2 off
*/

// _delay_loop_1(t); cannot be called from static inline function? arghh...
static inline void my_delay(uint8_t t)
{
	while(t) t--;
}

static inline void busy_square(void)
{
	uint16_t i;
	uint8_t k8;
	volatile uint8_t t;
	
	switcher_init();
	for(i=SWITCHER_CYCLES; i; i--){
		for(k8=250; k8; k8--){
			SWITCHER_PORT = SWITCHER_STATE1;
			t = SWITCHER_DELAY; my_delay(t);
			SWITCHER_PORT = SWITCHER_STATE2;
			t = SWITCHER_DELAY; my_delay(t);

			SWITCHER_PORT = SWITCHER_STATE1;
			t = SWITCHER_DELAY; my_delay(t);
			SWITCHER_PORT = SWITCHER_STATE2;
			t = SWITCHER_DELAY; my_delay(t);

			SWITCHER_PORT = SWITCHER_STATE1;
			t = SWITCHER_DELAY; my_delay(t);
			SWITCHER_PORT = SWITCHER_STATE2;
			t = SWITCHER_DELAY; my_delay(t);

			SWITCHER_PORT = SWITCHER_STATE1;
			t = SWITCHER_DELAY; my_delay(t);
			SWITCHER_PORT = SWITCHER_STATE2;
			t = SWITCHER_DELAY-1; my_delay(t);
		}
	}
	SWITCHER_PORT = SWITCHER_STATE0;
}
#endif

/*@}*/

