/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Michael Kristensen, mik@caffrey.dk]          *
***********************************************************
*  -- History --                                          *
*    release 0.1 - 06-03-2003                             *
*            http://caffrey.dk/megasquirt                 *
*    initial Motorola version by Bowling & Grippo (2002)  *
*            http://www.bgsoflex.com/megasquirt.html      *
\*********************************************************/

#ifndef VE_H
#define VE_H

// length of buffer used for statistics
#define STAT_SIZE 16

// number of samples from the buffer to evaluate
#define VE_LEARN_SAMPLES 16

// upon _entry_ of closed loop, the might still be some acc enrichment
// skip the first n*0.1 seconds
#define VE_LEARN_SKIP_SAMPLES 10

// how many entries in the VE-table
#ifndef VE_SIZE_RPM
#define VE_SIZE_RPM 8
#endif

#ifndef VE_SIZE_KPA
#define VE_SIZE_KPA 8
#endif

#define VE_SIZE VE_SIZE_RPM*VE_SIZE_KPA

#define LAMBDACORR_OFFSET (200)
#define LAMBDACORR_STOICH (256-LAMBDACORR_OFFSET)

struct ve_t {
  uint16_t ve_table[VE_SIZE_RPM*VE_SIZE_KPA]; // main VE table (modified by VE learning)
  uint8_t ref_table[VE_SIZE_RPM*VE_SIZE_KPA];  // reference VE table
  uint8_t lambda[VE_SIZE_RPM*VE_SIZE_KPA];  // lambdacorr table
  uint8_t rpm_range[VE_SIZE_RPM];  // VE table RPM bins for 2-D interpolation
  uint8_t kpa_range[VE_SIZE_KPA];  // VE table MAP bins for 2_D interpolation
};

extern struct ve_t ve;

// these are not inside struct ve_t since we do not want to store in eeprom, just SRAM
extern struct search_table_t kpa_idx, rpm_idx;
extern uint16_t rpm_rx21[VE_SIZE_RPM];	// the precalculated constant/(rpm_range[x]-rpm_range[x-1])
extern uint16_t kpa_rx21[VE_SIZE_KPA]; // likewise

// algorithm is (x-x1) * RX21_CONSTANT/(x2-x1) >> RX21_CONSTANT_EXP
#define RX21_CONSTANT_EXP	16
#define RX21_CONSTANT	0xFFFF

#ifndef NOIGN
// currently ignition shares the same bins as ve table (easier on the tuner person)
struct ign_t {
	uint8_t adv_table[VE_SIZE_RPM*VE_SIZE_KPA];  // ignition advance table: bigger is higher advance
};
extern struct ign_t ign;
#endif

#define HWMAP_SIZE 8
#define HWMAP_LINES (3)
#define HWMAP_BYTES (HWMAP_SIZE * HWMAP_LINES)
#ifdef HWMAP
// hardware mappings
struct hwmap_t {
	uint8_t table[HWMAP_BYTES];
};
extern struct hwmap_t hwmap;
#endif
struct ve_learn_stat_t {
  uint8_t rpm[STAT_SIZE];
  uint8_t kpa[STAT_SIZE];
  uint8_t ego[STAT_SIZE];
};

struct res_t {
  uint8_t mean;
  uint16_t variance;
};

struct ve_stat_res_t {
  struct res_t rpm;
  struct res_t kpa;
  struct res_t ego;
};

struct ve_at_t {  // ve_auto_tune_t
  uint8_t rpm;
  uint8_t kpa;
  uint8_t adjust;
  uint8_t confidence;
  uint8_t w00, w01, w10, w11;  // weight [low/high rpm] [low/high kpa]
  uint8_t cr, ck, ce;  // used for logging
};

extern struct ve_at_t tune_me;
extern struct ve_stat_res_t res;

void ve_auto_tune(struct ve_at_t *a);
void ve_entry_adjust(uint8_t rpm_idx, uint8_t kpa_idx, uint8_t weight, uint8_t adjust);
void ve_learn(void);
void table_lookup(void);
uint8_t ve_check_limit(uint8_t rpm, uint8_t kpa);
//void ve_calc_weights(uint8_t rpm_idx, uint8_t kpa_idx, struct ve_at_t *a);
void ve_calc_stat(uint8_t *data, struct res_t *res);
uint8_t ve_variance_scale(uint8_t val, uint8_t scale);

void sum_square(int8_t val, uint16_t *sum);
void copyVEtoVEref(void);
void copyVEreftoVE(void);
void search_rpm_kpa_index(void);

uint16_t degree2time(uint16_t d);

#endif
